@@ -1,6 +1,6 @@
 /***************************************************************************
- * SPDX-FileCopyrightText: 2021 S. MANKOWSKI stephane@mankowski.fr
- * SPDX-FileCopyrightText: 2021 G. DE BURE support@mankowski.fr
+ * SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
+ * SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
  * SPDX-License-Identifier: GPL-3.0-or-later
  ***************************************************************************/
 #ifndef SKGSERVICES_H
@@ -17,6 +17,8 @@
 #include <qicon.h>
 #include <qstringlist.h>
 
+#include <kpluginmetadata.h>
+
 #include "skgbasemodeler_export.h"
 #include "skgdefine.h"
 #include "skgerror.h"
@@ -55,8 +57,8 @@ using SKGStringListList = QList<QStringList>;
 using SKGIntList = QList<int>;
 
 /**
-* This class containing various static services
-*/
+ * This class containing various static services
+ */
 class SKGBASEMODELER_EXPORT SKGServices : public QObject
 {
     Q_OBJECT
@@ -92,16 +94,16 @@ public:
      * This structure defines attributes types
      */
     enum AttributeType {
-        TEXT,       /**< Text */
-        INTEGER,    /**< Integer */
-        FLOAT,      /**< Float */
-        DATE,       /**< Date */
-        ID,         /**< Unique identifier */
-        LINK,       /**< Link */
-        BLOB,       /**< Blob */
-        BOOL,       /**< Boolean (Y, N) */
-        TRISTATE,   /**< Tri-state (N, P, C) */
-        OTHER           /**< Other or undefined */
+        TEXT, /**< Text */
+        INTEGER, /**< Integer */
+        FLOAT, /**< Float */
+        DATE, /**< Date */
+        ID, /**< Unique identifier */
+        LINK, /**< Link */
+        BLOB, /**< Blob */
+        BOOL, /**< Boolean (Y, N) */
+        TRISTATE, /**< Tri-state (N, P, C) */
+        OTHER /**< Other or undefined */
     };
 
     /**
@@ -113,20 +115,20 @@ public:
      * Describe an attribute
      */
     struct SKGAttributeInfo {
-        QString name;           /**< The internal name of the attribute */
-        QString display;        /**< The name for display of the attribute */
-        QIcon icon;                 /**< The icon for display of the attribute */
-        AttributeType type;         /**< The type of the attribute */
-        bool notnull{};           /**< To know if the attribute can be null or not */
-        QString defaultvalue;       /**< The default value of the attribute */
+        QString name; /**< The internal name of the attribute */
+        QString display; /**< The name for display of the attribute */
+        QIcon icon; /**< The icon for display of the attribute */
+        AttributeType type; /**< The type of the attribute */
+        bool notnull{}; /**< To know if the attribute can be null or not */
+        QString defaultvalue; /**< The default value of the attribute */
     };
 
     /**
      * Search criteria
      */
     struct SKGSearchCriteria {
-        QChar mode;              /**< The mode '+' or '-' */
-        QStringList words;       /**< The list of words */
+        QChar mode; /**< The mode '+' or '-' */
+        QStringList words; /**< The list of words */
     };
 
     /**
@@ -144,7 +146,7 @@ public:
      */
     enum DumpMode {
         DUMP_CSV, /**< To dump in CSV mode */
-        DUMP_TEXT  /**< To dump in DUMP_TEXT mode */
+        DUMP_TEXT /**< To dump in DUMP_TEXT mode */
     };
 
     /**
@@ -157,7 +159,7 @@ public:
      * @param iString a string to analyze
      * @return the search criteria list
      */
-    static SKGServices::SKGSearchCriteriaList stringToSearchCriterias(const QString& iString);
+    static SKGServices::SKGSearchCriteriaList stringToSearchCriterias(const QString &iString);
 
     /**
      * Compute a SQL where clause with criteria and list of attributes
@@ -167,17 +169,17 @@ public:
      * @param iForDisplay to build the human readable string explaining the query
      * @return the where clause
      */
-    static QString searchCriteriasToWhereClause(const SKGServices::SKGSearchCriteriaList& iSearchCriterias,
-            const QStringList& iAttributes,
-            const SKGDocument* iDocument,
-            bool iForDisplay = false);
+    static QString searchCriteriasToWhereClause(const SKGServices::SKGSearchCriteriaList &iSearchCriterias,
+                                                const QStringList &iAttributes,
+                                                const SKGDocument *iDocument,
+                                                bool iForDisplay = false);
 
     /**
      * Get environment variable
      * @param iAttribute name of the variable
      * @return value of the variable
      */
-    static QString getEnvVariable(const QString& iAttribute);
+    static QString getEnvVariable(const QString &iAttribute);
 
     /**
      * Convert a integer into a QString
@@ -191,14 +193,14 @@ public:
      * @param iNumber the QString to convert
      * @return the converted integer
      */
-    static qlonglong stringToInt(const QString& iNumber);
+    static qlonglong stringToInt(const QString &iNumber);
 
     /**
      * Convert a QString into a csv QString
      * @param iNumber the QString to convert
      * @return the converted csv string
      */
-    static QString stringToCsv(const QString& iNumber);
+    static QString stringToCsv(const QString &iNumber);
 
     /**
      * Convert a QStringList into a csv QString
@@ -206,21 +208,21 @@ public:
      * @param iSeparator the separator
      * @return the converted csv string
      */
-    static QString stringsToCsv(const QStringList& iList, QChar iSeparator = QLatin1Char(';'));
+    static QString stringsToCsv(const QStringList &iList, QChar iSeparator = u';');
 
     /**
      * Convert a QString to a html QString
      * @param iString the QString to convert
      * @return the converted html string
      */
-    static QString stringToHtml(const QString& iString);
+    static QString stringToHtml(const QString &iString);
 
     /**
      * Convert a html QString to a QString
      * @param iString the QString to convert
      * @return the converted string
      */
-    static QString htmlToString(const QString& iString);
+    static QString htmlToString(const QString &iString);
 
     /**
      * Convert a double into a QString
@@ -232,9 +234,10 @@ public:
     /**
      * Convert a QString into an double
      * @param iNumber the QString to convert
+     * @param iUseLocale try to use the locale
      * @return the converted double
      */
-    static double stringToDouble(const QString& iNumber);
+    static double stringToDouble(const QString &iNumber, bool iUseLocale = false);
 
     /**
      * Convert a QDateTime into a QString ready for sql order.
@@ -242,7 +245,7 @@ public:
      * @param iDateTime the time
      * @return the converted QString
      */
-    static QString timeToString(const QDateTime& iDateTime);
+    static QString timeToString(const QDateTime &iDateTime);
 
     /**
      * Convert a date in a period
@@ -257,7 +260,7 @@ public:
      * @return the period string
      */
     // cppcheck-suppress passedByValue
-    static QString dateToPeriod(QDate iDate, const QString& iPeriod);
+    static QString dateToPeriod(QDate iDate, const QString &iPeriod);
 
     /**
      * Compute the number of working days between two dates.
@@ -273,7 +276,14 @@ public:
      * @param iDateString the time. The format must be "%Y-%m-%d %H-%M-%S" or "%Y-%m-%d"
      * @return the converted QDateTime
      */
-    static QDateTime stringToTime(const QString& iDateString);
+    static QDateTime stringToTime(const QString &iDateString);
+
+    /**
+     * Convert a QString to QDate.
+     * @param iDateString the date. The format must be "%Y-%m-%d"
+     * @return the converted QDate
+     */
+    static QDate stringToDate(const QString &iDateString);
 
     /**
      * Convert a QString to QDate.
@@ -281,7 +291,7 @@ public:
      * @param iFixupBackward the date string is fixed in backward mode.
      * @return the converted QDate
      */
-    static QDate partialStringToDate(const QString& iDateString, bool iFixupBackward = true);
+    static QDate partialStringToDate(const QString &iDateString, bool iFixupBackward = true);
 
     /**
      * Convert a QDateTime into a QString ready for sql order.
@@ -289,7 +299,7 @@ public:
      * @param iDateTime the time
      * @return the converted QString
      */
-    static QString dateToSqlString(const QDateTime& iDateTime);
+    static QString dateToSqlString(const QDateTime &iDateTime);
 
     /**
      * Convert a QDate into a QString ready for sql order.
@@ -312,7 +322,7 @@ public:
      * "DD/MMM/YYYY" "DD-MMM-YYYY" DDMMMYYYY
      * @return the converted QString
      */
-    static QString dateToSqlString(const QString& iDate, const QString& iFormat);
+    static QString dateToSqlString(const QString &iDate, const QString &iFormat);
 
     /**
      * Get the sql where clause corresponding to a period date.
@@ -321,14 +331,15 @@ public:
      * @param iComparator the sql comparator (= by default. But you can use < or >)
      * @return the where clause
      */
-    static QString getPeriodWhereClause(const QString& iPeriod, const QString& iDateAttribute = QStringLiteral("d_date"), const QString& iComparator = QStringLiteral("="));
+    static QString
+    getPeriodWhereClause(const QString &iPeriod, const QString &iDateAttribute = QStringLiteral("d_date"), const QString &iComparator = QStringLiteral("="));
 
     /**
      * Get the last date of a period.
      * @param iPeriod the period date
      * @return the last date
      */
-    static QDate periodToDate(const QString& iPeriod);
+    static QDate periodToDate(const QString &iPeriod);
 
     /**
      * Get the neighboring period.
@@ -336,14 +347,14 @@ public:
      * @param iDelta the delta to apply (-1 = previous, +1 = next)
      * @return the neighboring period
      */
-    static QString getNeighboringPeriod(const QString& iPeriod, int iDelta = -1);
+    static QString getNeighboringPeriod(const QString &iPeriod, int iDelta = -1);
 
     /**
      * Get the format of a list of string dates
      * @param iDates a list of string dates
      * @return the supported format of all of these dates.
      */
-    static QString getDateFormat(const QStringList& iDates);
+    static QString getDateFormat(const QStringList &iDates);
 
     /**
      * Get the string representing the percentage
@@ -360,24 +371,25 @@ public:
      * @param iNbDecimal the number of decimals
      * @return the string.
      */
-    static QString toCurrencyString(double iAmount, const QString& iSymbol = QString(), int iNbDecimal = 2);
+    static QString toCurrencyString(double iAmount, const QString &iSymbol = QString(), int iNbDecimal = 2);
 
     /**
      * Return string ready for sql order
      * @param iString the string
      * @return the escaped string
      */
-    static QString stringToSqlString(const QString& iString);
+    static QString stringToSqlString(const QString &iString);
 
     /**
      * Split a csv line
      * @param iString the csv line
      * @param iSeparator the csv separator
      * @param iCoteDefineBlock to indicate if the character double cote define a block
-     * @param oRealSeparator the real csv separator. If the csv string is strictly all enclosed then oRealSeparator will contain the real separator. Can be nullptr
+     * @param oRealSeparator the real csv separator. If the csv string is strictly all enclosed then oRealSeparator will contain the real separator. Can be
+     * nullptr
      * @return the list of all values in this csv line. Return an empty list if some cote are opened and not closed
      */
-    static QStringList splitCSVLine(const QString& iString, QChar iSeparator, bool iCoteDefineBlock, QChar* oRealSeparator);
+    static QStringList splitCSVLine(const QString &iString, QChar iSeparator, bool iCoteDefineBlock, QChar *oRealSeparator);
 
     /**
      * Split a csv line
@@ -386,7 +398,7 @@ public:
      * @param iCoteDefineBlock to indicate if the character double cote define a block
      * @return the list of all values in this csv line. Return an empty list if some cote are opened and not closed
      */
-    static QStringList splitCSVLine(const QString& iString, QChar iSeparator = QLatin1Char(';'), bool iCoteDefineBlock = true);
+    static QStringList splitCSVLine(const QString &iString, QChar iSeparator = u';', bool iCoteDefineBlock = true);
 
     /**
      * To dump a table
@@ -394,14 +406,14 @@ public:
      * @param iMode dump mode
      * @return the dump of the table
      */
-    static QStringList tableToDump(const SKGStringListList& iTable, SKGServices::DumpMode iMode);
+    static QStringList tableToDump(const SKGStringListList &iTable, SKGServices::DumpMode iMode);
 
     /**
      * Get the table name corresponding to a table or a view
      * @param iTable table or view
      * @return table name corresponding
      */
-    static QString getRealTable(const QString& iTable);
+    static QString getRealTable(const QString &iTable);
 
     /**
      * Copy a sqlite database from memory to file or from file to memory.
@@ -415,16 +427,18 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError copySqliteDatabase(const QSqlDatabase& iFileDb, const QSqlDatabase& iMemoryDb, bool iFromFileToMemory, const QString& iPassword = QString());
+    static SKGError
+    copySqliteDatabase(const QSqlDatabase &iFileDb, const QSqlDatabase &iMemoryDb, bool iFromFileToMemory, const QString &iPassword = QString());
 
     /**
      * Copy a sqlite database into an XML document.
      * @param iDb the sqlite pointer corresponding to database
      * @param oDocument the xml document
+     * @param oDocument the xml document
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError copySqliteDatabaseToXml(const QSqlDatabase& iDb, QDomDocument& oDocument);
+    static SKGError copySqliteDatabaseToXml(const QSqlDatabase &iDb, QDomDocument &oDocument, QVector<QString> *iIgnore = nullptr);
 
     /**
      * Execute a sqlite orders
@@ -433,7 +447,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSqliteOrders(const QSqlDatabase& iDb, const QStringList& iSqlOrders);
+    static SKGError executeSqliteOrders(const QSqlDatabase &iDb, const QStringList &iSqlOrders);
 
     /**
      * Execute a sqlite order
@@ -444,7 +458,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, const QMap<QString, QVariant>& iBind, int* iLastId);
+    static SKGError executeSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, const QMap<QString, QVariant> &iBind, int *iLastId);
 
     /**
      * Execute a sqlite order
@@ -454,17 +468,17 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, int* iLastId = nullptr);
+    static SKGError executeSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, int *iLastId = nullptr);
 
     /**
-    * Execute a select sqlite order and return the result in @p oResult
-    * @param iDb A database pointer
-    * @param iSqlOrder the sql order
-    * @param oResult the result of the select. It is a vector of vector of QString
-    * @return An object managing the error
-    *   @see SKGError
-    */
-    static SKGError executeSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, SKGStringListList& oResult);
+     * Execute a select sqlite order and return the result in @p oResult
+     * @param iDb A database pointer
+     * @param iSqlOrder the sql order
+     * @param oResult the result of the select. It is a vector of vector of QString
+     * @return An object managing the error
+     *   @see SKGError
+     */
+    static SKGError executeSelectSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, SKGStringListList &oResult);
 
     /**
      * Execute a select sqlite order returning one value and return the result in @p oResult
@@ -474,28 +488,28 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSingleSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QString& oResult);
+    static SKGError executeSingleSelectSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, QString &oResult);
 
     /**
-    * dump a select sqlite order
-    * @param iDb A database pointer
-    * @param iSqlOrder the sql order
-    * @param iMode dump mode
-    * @return An object managing the error
-    *   @see SKGError
-    */
-    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, SKGServices::DumpMode iMode = DUMP_TEXT);
+     * dump a select sqlite order
+     * @param iDb A database pointer
+     * @param iSqlOrder the sql order
+     * @param iMode dump mode
+     * @return An object managing the error
+     *   @see SKGError
+     */
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, SKGServices::DumpMode iMode = DUMP_TEXT);
 
     /**
-    * dump a select sqlite order
-    * @param iDb A database pointer
-    * @param iSqlOrder the sql order
-    * @param oStream the output stream, nullptr for std output (cout)
-    * @param iMode dump mode
-    * @return An object managing the error
-    *   @see SKGError
-    */
-    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QTextStream* oStream, SKGServices::DumpMode iMode = DUMP_TEXT);
+     * dump a select sqlite order
+     * @param iDb A database pointer
+     * @param iSqlOrder the sql order
+     * @param oStream the output stream, nullptr for std output (cout)
+     * @param iMode dump mode
+     * @return An object managing the error
+     *   @see SKGError
+     */
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, QTextStream *oStream, SKGServices::DumpMode iMode = DUMP_TEXT);
 
     /**
      * dump a select sqlite order
@@ -506,7 +520,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QString& oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, QString &oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
 
     /**
      * dump a select sqlite order
@@ -517,7 +531,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QStringList& oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase &iDb, const QString &iSqlOrder, QStringList &oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
 
     /**
      * Read a property file
@@ -526,7 +540,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError readPropertyFile(const QString& iFileName, QHash<QString, QString>& oProperties);
+    static SKGError readPropertyFile(const QString &iFileName, QHash<QString, QString> &oProperties);
 
     /**
      * Return the current time in millisecond
@@ -545,12 +559,12 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError cryptFile(const QString& iFileSource,
-                              const QString& iFileTarget,
-                              const QString& iPassword,
+    static SKGError cryptFile(const QString &iFileSource,
+                              const QString &iFileTarget,
+                              const QString &iPassword,
                               bool iEncrypt,
-                              const QString& iHeaderFile,
-                              bool& oModeSQLCipher);
+                              const QString &iHeaderFile,
+                              bool &oModeSQLCipher);
 
     /**
      * Download a non local URL to a string
@@ -559,7 +573,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError downloadToStream(const QUrl& iSourceUrl, QByteArray& oStream);
+    static SKGError downloadToStream(const QUrl &iSourceUrl, QByteArray &oStream);
 
     /**
      * Download a non local URL to a temporary file
@@ -568,7 +582,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError download(const QUrl& iSourceUrl, QString& oTemporaryFile);
+    static SKGError download(const QUrl &iSourceUrl, QString &oTemporaryFile);
 
     /**
      * Upload from a source to a destination
@@ -577,7 +591,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError upload(const QUrl& iSourceUrl, const QUrl& iDescUrl);
+    static SKGError upload(const QUrl &iSourceUrl, const QUrl &iDescUrl);
 
     /**
      * To enable, disable sql traces
@@ -591,14 +605,14 @@ public:
      * @param iAbsolute transform all values in absolute
      * @return the table
      */
-    static SKGStringListList getPercentTable(const SKGStringListList& iTable, bool iOfColumns = true, bool iAbsolute = false);
+    static SKGStringListList getPercentTable(const SKGStringListList &iTable, bool iOfColumns = true, bool iAbsolute = false);
 
     /**
      * Create a table in base 100
      * @param iTable table
      * @return the table
      */
-    static SKGStringListList getBase100Table(const SKGStringListList& iTable);
+    static SKGStringListList getBase100Table(const SKGStringListList &iTable);
 
     /**
      * Create an historized table.
@@ -606,14 +620,14 @@ public:
      * @param iTable table
      * @return the table
      */
-    static SKGStringListList getHistorizedTable(const SKGStringListList& iTable);
+    static SKGStringListList getHistorizedTable(const SKGStringListList &iTable);
 
     /**
      * Encode a string for an url
      * @param iString the decoded string
      * @return the encoded string
      */
-    static QString encodeForUrl(const QString& iString);
+    static QString encodeForUrl(const QString &iString);
 
     /**
      * Return the icon form theme with expected overlays
@@ -622,28 +636,35 @@ public:
      * @param iOverlays List of overlays
      * @return The icon
      */
-    static QIcon fromTheme(const QString& iName, const QStringList& iOverlays = QStringList());
+    static QIcon fromTheme(const QString &iName, const QStringList &iOverlays = QStringList());
 
     /**
      * Get a major version (ex: 4.3) of a version (ex: 4.3.1)
      * @param iVersion the version
      * @return the major version
      */
-    static QString getMajorVersion(const QString& iVersion);
+    static QString getMajorVersion(const QString &iVersion);
 
     /**
      * Get a full path command line
      * @param iCommandLine command line
      * @return the full path command line
      */
-    static QString getFullPathCommandLine(const QString& iCommandLine);
+    static QString getFullPathCommandLine(const QString &iCommandLine);
 
     /**
      * Get the next string
      * @param iString the string
      * @return the next string
      */
-    static QString getNextString(const QString& iString);
+    static QString getNextString(const QString &iString);
+
+    /**
+     * Search data plugin
+     * @param type the type of plugins
+     * @return the plugins found
+     */
+    static QVector<KPluginMetaData> findDataPlugins(const QString &type);
 
 private:
     Q_DISABLE_COPY(SKGServices)
